<?php

/*  PEL: PHP Exif Library.  A library with support for reading and
 *  writing all Exif headers in JPEG and TIFF images using PHP.
 *
 *  Copyright (C) 2004, 2005  Martin Geisler.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program in the file COPYING; if not, write to the
 *  Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 *  Boston, MA 02110-1301 USA
 */

/* $Id: PelException.php 10238 2011-12-14 08:09:07Z giangnd $ */


/**
 * Standard PEL exception.
 *
 * @author Martin Geisler <mgeisler@users.sourceforge.net>
 * @version $Revision: 396 $
 * @date $Date: 2005-10-24 00:36:10 +0200 (Mon, 24 Oct 2005) $
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public
 * License (GPL)
 * @package PEL
 */

/**
 * A printf() capable exception.
 *
 * This class is a simple extension of the standard Exception class in
 * PHP, and all the methods defined there retain their original
 * meaning.
 *
 * @package PEL
 * @subpackage Exception
 */
class PelException extends Exception {

  /**
   * Construct a new PEL exception.
   *
   * @param string $fmt an optional format string can be given.  It
   * will be used as a format string for vprintf().  The remaining
   * arguments will be available for the format string as usual with
   * vprintf().
   *
   * @param mixed $args,... any number of arguments to be used with
   * the format string.
   */
  function __construct(/* fmt, args... */) {
    $args = func_get_args();
    $fmt = array_shift($args);
    parent::__construct(vsprintf($fmt, $args));
  }
}


/**
 * Exception throw if invalid data is found.
 *
 * @author Martin Geisler <mgeisler@users.sourceforge.net>
 * @package PEL
 * @subpackage Exception
 */
class PelInvalidDataException extends PelException {}

/**
 * Exception throw if an invalid argument is passed.
 *
 * @author Martin Geisler <mgeisler@users.sourceforge.net>
 * @package PEL
 * @subpackage Exception
 */
class PelInvalidArgumentException extends PelException {}

?>