<?php
/**
* @author Fastball Productions
* @package com_fullcourt
* @subpackage components
* @link http://www.fastballproductions.com
* @copyright Copyright (C) 2009-2017 Fastball Productions
* @license http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
*/

/* no direct access; */
defined( '_JEXEC' ) or die( 'Restricted access' );

jimport( 'joomla.application.component.model' );

class FullcourtModelLeagues extends JModelLegacy {
	var $_leagues;
	var $_divisions;
	var $_total = null;
	var $_pagination = null;

	function __construct() {
		$app = JFactory::getApplication();
		$option = JRequest::getCmd('option');
		parent::__construct();
		$limit = $app->getUserStateFromRequest('global.list.limit', 'limit', $app->getCfg('list_limit'));
		$limitstart = $app->getUserStateFromRequest($option.'limitstart', 'limitstart', 0);
		$this->setState('limit', $limit);
		$this->setState('limitstart', $limitstart);
	}

	function saveOrder() {
		$db = $this->getDBO();

		$cid = JRequest::getVar( 'cid', array(), 'post', 'array' );
		$order = JRequest::getVar( 'order', array(), 'post', 'array' );

		$row =  JTable::getInstance('league', 'Table');
		for ($i = 0; $i < sizeof($cid); $i++) {
			$row->load((int)$cid[$i]);
			if ($row->ordering != $order[$i]) {
				$row->ordering = $order[$i];
				if (!$row->store()) {
					return JError::raiseError( 500, $db->getErrorMsg() );
				}
			}
		}

		return true;
	}

	function getLeagues() {
		$db  =  $this->getDBO();
		$limitstart = $this->getState('limitstart');
		$limit = $this->getState('limit');
		$query = "SELECT * FROM #__fullcourt_league ORDER BY ordering ASC";
		$this->_leagues = $this->_getList($query, $limitstart, $limit);
		$this->_total = $this->_getListCount($query);
		if(empty($this->_leagues) && $this->_total > 0){
			$this->_leagues = $this->_getList($query, 0, $limit);
		}
		return $this->_leagues;
	}

	function getDivisions() {
		$db  =  $this->getDBO();
		$query = "SELECT * FROM #__fullcourt_division ORDER BY leagueid ASC, division ASC";
		$db->setQuery($query);
		$this->_divisions = $db->loadObjectList();
		return $this->_divisions;
	}

	function delete($cids) {
		$db = $this->getDBO();
		$table = $db->quoteName('#__fullcourt_league');
		$id = $db->quoteName('id');
		$query = "DELETE FROM " . $table . " WHERE " . $id . "IN (" . implode( ',', $cids ) . ")";
		$db->setQuery( $query );

		if(!$db->query()) {
			$errorMessage = $this->getDBO()->getErrorMsg();
			JError::raiseError( 500, 'Error deleting revues: ' . $errorMessage );
		}
	}

	function deleteDivision($cids, $leagueid) {
		$db = $this->getDBO();
		$table = $db->quoteName('#__fullcourt_division');

		/* get the leagueid so we know what league is being edited; */
		$leagueid = $db->quoteName('leagueid');
		$id = $db->quoteName('id');
		$query = "DELETE FROM " . $table . " WHERE " . $id . "IN (" . implode( ',', $cids ) . ")";
		$db->setQuery($query);

		if (!$db->query()) {
			$errorMessage = $this->getDBO()->getErrorMsg();
			JError::raiseError( 500, 'Error deleting revues: ' . $errorMessage );
		}

		/* update all teams and set their divisionid to 0 where the division is one that was deleted; */
		$table = $db->quoteName('#__fullcourt_team');
		$divisionid = $db->quoteName('divisionid');
		$query = "UPDATE " . $table . " SET " . $divisionid . " = 0 WHERE " . $divisionid . "IN (" . implode( ',', $cids ) . ")";
		$db->setQuery($query);
		$db->query();
	}

	function publish() {
		$db  =  $this->getDBO();
		$cid = JRequest::getVar( 'cid', array(0), '', 'array' );
		$task = JFactory::getApplication()->input->get( 'task', '', 'default' );
		if ($task == 'leaguepublish') {
			$publish = 1;
		}
		else {
			$publish = 0;
		}
		$table =  JTable::getInstance('league', 'Table');
		$table->publish($cid, $publish);
	}

	function getPagination() {
		if (empty($this->_pagination)) {
			jimport('joomla.html.pagination');
			$total = $this->_total;
			$limitstart = $this->getState('limitstart');
			$limit = $this->getState('limit');
			$this->_pagination = new JPagination($total, $limitstart, $limit);
		}
		return $this->_pagination;
	}

}

?>