<?php
/**
* @author Fastball Productions
* @package com_fullcourt
* @subpackage components
* @link http://www.fastballproductions.com
* @copyright Copyright (C) 2009-2017 Fastball Productions
* @license http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
*/

/* no direct access; */
defined('_JEXEC') or die('Restricted access');

jimport('joomla.application.component.model');

class FullcourtModelTeams extends JModelLegacy {
	var $_teams;
	var $_total = null;
	var $_lists = null;
	var $_pagination = null;

	function __construct() {
		$app = JFactory::getApplication();
		$option = JRequest::getCmd('option');
		parent::__construct();
		$this->_lists = array();
		$limit = $app->getUserStateFromRequest('global.list.limit', 'limit', $app->getCfg('list_limit'));
		$limitstart = $app->getUserStateFromRequest($option.'limitstart', 'limitstart', 0);
		$this->setState('limit', $limit);
		$this->setState('limitstart', $limitstart);
	}

	function getTeams() {
		$app = JFactory::getApplication();
		$option = JRequest::getCmd('option');
		$db  =  $this->getDBO();

		/* get the search filter if provided; */
		$search = JRequest::getVar( 'tp_search', '', 'default', 'default' );

		/* get the league/division filter if applied; */
		$filterdivision = $app->getUserStateFromRequest($option . 'filter_division_teams', 'filter_division', '' );
		/* get the managed/unmanaged filter if applied; */
		$filtermanaged = $app->getUserStateFromRequest($option . 'filter_managed_teams', 'filter_managed', '' );

		$limitstart = $this->getState('limitstart');
		$limit = $this->getState('limit');

		$this->_lists['filterdivision'] = getLeagueDivisionFilterList('filter_division', $filterdivision);
		$this->_lists['filtermanaged'] = getManagedUnmanagedFilterList('filter_managed', $filtermanaged);

		$whereclause = array();
		if ($search) {
			$whereclause = "WHERE a.name LIKE '%$search%'";
		}
		else {
			if ($filterdivision != '') {
				$arr = explode(':', $filterdivision);

				if ($arr[0]) {
					$whereclause[] = "l.id = {$arr[0]}";
				}
				if ($arr[1]) {
					$whereclause[] = "d.id = {$arr[1]}";
				}
			}

			if ($filtermanaged != '') {
				$whereclause[] = "a.defaultteam = {$filtermanaged}";
			}

			if (sizeof($whereclause) > 0) {
				$whereclause = implode(' AND ', $whereclause);
				$whereclause = 'WHERE (' . $whereclause;
				$whereclause .= ')';
			}
			else {
				$whereclause = '';
			}
		}

		$query = "SELECT a.*, l.name AS league, d.division FROM #__fullcourt_team AS a LEFT JOIN #__fullcourt_league AS l ON a.leagueid = l.id LEFT JOIN #__fullcourt_division AS d ON a.divisionid = d.id {$whereclause} GROUP BY a.id ORDER BY a.name";
		$this->_teams = $this->_getList($query, $limitstart, $limit);
		$this->_total = $this->_getListCount($query);
		if(empty($this->_teams) && $this->_total > 0){
			$this->_teams = $this->_getList($query, 0, $limit);
		}
		return $this->_teams;
	}


	function delete($cids) {
		$db = $this->getDBO();
		$table = $db->quoteName('#__fullcourt_team');
		$id = $db->quoteName('id');
		$query = "DELETE FROM " . $table . " WHERE " . $id . "IN (" . implode( ',', $cids ) . ")";
		$db->setQuery( $query );

		if (!$db->query()) {
			$errorMessage = $this->getDBO()->getErrorMsg();
			JError::raiseError( 500, 'Error deleting team: ' . $errorMessage );
		}

		if (sizeof($cids)) {
			foreach ($cids as $cid) {
				/* get a list of all the games associated with this team so they can be removed; */
				$db->setQuery("SELECT id FROM #__fullcourt_schedule WHERE (visitingteam = {$cid} OR hometeam = {$cid})");
				$games = $db->loadObjectList();
				/* delete linescores table records; */
				foreach ($games as $game) {
					$db->setQuery("DELETE FROM #__fullcourt_linescore WHERE (gameid = {$game->id})");
					$db->query();
				}
				/* delete the schedule table records; */
				$db->setQuery("DELETE FROM #__fullcourt_schedule WHERE (visitingteam = {$cid} OR hometeam = {$cid})");
				$db->query();
			}
		}
	}

	function publish() {
		$db  =  $this->getDBO();
		$cid = JRequest::getVar( 'cid', array(0), '', 'array' );
		$task = JRequest::getVar( 'task', '', 'default', 'default' );
		if ($task == 'teampublish') {
			$publish = 1;
		}
		else {
			$publish = 0;
		}
		$table =  JTable::getInstance('team', 'Table');
		$table->publish($cid, $publish);
	}

	function getPagination() {
		if (empty($this->_pagination)) {
			jimport('joomla.html.pagination');
			$total = $this->_total;
			$limitstart = $this->getState('limitstart');
			$limit = $this->getState('limit');
			$this->_pagination = new JPagination($total, $limitstart, $limit);
		}
		return $this->_pagination;
	}

}

?>