<?php
/**
* @author Fastball Productions
* @package com_fullcourt
* @subpackage components
* @link http://www.fastballproductions.com
* @copyright Copyright (C) 2009-2017 Fastball Productions
* @license http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
*/

/* no direct access; */
defined('_JEXEC') or die('Restricted access');

jimport('joomla.application.component.model');

class FullcourtModelPlayers extends JModelLegacy {
	var $_players;
	var $_total = null;
	var $_lists = null;
	var $_pagination = null;

	function __construct() {
		$app = JFactory::getApplication();
		$option = JRequest::getCmd('option');
		parent::__construct();
		$this->_lists = array();
		$limit = $app->getUserStateFromRequest('global.list.limit', 'limit', $app->getCfg('list_limit'));
		$limitstart = $app->getUserStateFromRequest($option.'limitstart', 'limitstart', 0);
		$this->setState('limit', $limit);
		$this->setState('limitstart', $limitstart);
	}

	function getPlayers() {
		$option = JRequest::getCmd('option');
		$app = JFactory::getApplication();
		$db  =  $this->getDBO();

		/* get the search filter if provided; */
		$search = JRequest::getVar( 'tp_search', '', 'default', 'default' );

		/* get the league/division filter if applied; */
		$filterteam = $app->getUserStateFromRequest($option . 'filter_player_teams', 'filter_team', '' );
		/* get the managed/unmanaged filter if applied; */
		$filterpublished = $app->getUserStateFromRequest($option . 'filter_player_published', 'filter_published', '' );

		$limitstart = $this->getState('limitstart');
		$limit = $this->getState('limit');

		$this->_lists['filterteam'] = getTeamFilterList('filter_team', $filterteam);
		$this->_lists['filterpublished'] = getPlayerStatusFilterList('filter_published', $filterpublished);

		/* build the filtering option for the database lookup; */
		$whereclause = array();
		if ($search) {
			$whereclause = "a.fname LIKE '%$search%' || a.lname LIKE '%$search%'";
		}
		else {
			if ($filterteam != '') {
				$whereclause[] = "a.teamid = {$filterteam}";
			}
			if ($filterpublished != '') {
				$whereclause[] = "a.published = {$filterpublished}";
			}
			$whereclause = implode(' AND ', $whereclause);
		}

		if ($whereclause != '') {
			$query = "SELECT a.*, CONCAT(a.lname, ', ', a.fname, ' ', a.mname) AS fullname, t.name AS teamname, l.name AS league, d.division AS division FROM #__fullcourt_player AS a LEFT JOIN #__fullcourt_team AS t ON a.teamid = t.id LEFT JOIN #__fullcourt_league AS l ON t.leagueid = l.id LEFT JOIN #__fullcourt_division AS d ON t.divisionid = d.id WHERE ($whereclause) GROUP BY a.id ORDER BY a.teamid, a.lname";
		}
		else {
			$query = "SELECT a.*, CONCAT(a.lname, ', ', a.fname, ' ', a.mname) AS fullname, t.name AS teamname, l.name AS league, d.division AS division FROM #__fullcourt_player AS a LEFT JOIN #__fullcourt_team AS t ON a.teamid = t.id LEFT JOIN #__fullcourt_league AS l ON t.leagueid = l.id LEFT JOIN #__fullcourt_division AS d ON t.divisionid = d.id GROUP BY a.id ORDER BY a.teamid, a.lname";
		}
		$this->_players = $this->_getList($query, $limitstart, $limit);

		$this->_total = $this->_getListCount($query);
		
		if(empty($this->_players) && $this->_total > 0){
			$this->_players = $this->_getList($query, 0, $limit);
		}
		return $this->_players;
	}

	function delete($cids) {
		$db = $this->getDBO();
		$table = $db->quoteName('#__fullcourt_player');
		$id = $db->quoteName('id');
		$query = "DELETE FROM " . $table . " WHERE " . $id . "IN (" . implode( ',', $cids ) . ")";
		$db->setQuery($query);

		if(!$db->query()) {
			$errorMessage = $this->getDBO()->getErrorMsg();
			JError::raiseError( 500, 'Error deleting players: ' . $errorMessage );
		}

		/* delete player stats as well; */
		/*$table = $db->quoteName('#__fullcourt_playerstat');
		$playerid = $db->quoteName('playerid');
		$query = "DELETE FROM " . $table . " WHERE " . $playerid . "IN (" . implode( ',', $cids ) . ")";
		$db->setQuery($query);
		$db->query();*/
	}

	function publish() {
		$db  =  $this->getDBO();
		$cid = JRequest::getVar( 'cid', array(0), '', 'array' );
		$task = JRequest::getVar( 'task', '', 'default', 'default' );
		if ($task == 'playerpublish') {
			$publish = 1;
		}
		else {
			$publish = 0;
		}
		$table =  JTable::getInstance('player', 'Table');
		$table->publish($cid, $publish);
	}

	function getPagination() {
		if (empty($this->_pagination)) {
			jimport('joomla.html.pagination');
			$total = $this->_total;
			$limitstart = $this->getState('limitstart');
			$limit = $this->getState('limit');
			$this->_pagination = new JPagination($total, $limitstart, $limit);
		}
		return $this->_pagination;
	}

}

?>