<?php
/**
* @author Fastball Productions
* @package com_fullcourt
* @copyright Copyright (C) 2009-2017 Fastball Productions
* @license http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
*/

/* Change History 
  07/21/18 MCrosbie - Added support for game end time.
  08/11/18 MCrosbie - Added defaultteam and divisionid to adminteams SELECT statement.
*/

defined( '_JEXEC' ) or die( 'Restricted access' );

require_once(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_fullcourt' . DS . 'helpers' . DS . 'functions.php');

jimport( 'joomla.application.component.model' );

class ModelFullcourtAdmin extends JModelLegacy {
	var $_id = null;
	var $_team = null;
	var $_roster = null;
	var $_player = null;
	var $_playerid = null;
	var $_schedule = null;
	var $_game = null;
	var $_games = null;
	var $_gameinfo = null;
	var $_locations = null;
	var $_location = null;
	var $_fieldnames = null;
	var $_return = null;
	var $_teams = null;
	var $_teamname = null;
	var $_teamrow = null;
	var $_leagues = null;
	var $_adminteams = null;
	var $_teamid = null;
	var $_lists = null;
	var $_linescore = null;
	var $_boxscore = null;
	var $_playerstats = null;
	var $_hometeam = null;
	var $_visitingteam = null;
	var $_overtimev = null;
	var $_overtimeh = null;

	function __construct() {
		parent::__construct();

		/* get the user object; */
		$user = JFactory::getUser();

		/* get the team id to manage; */
		$session =  JFactory::getSession();
		$teamid = $session->get('_teamid');
		
		/* OST Changes */
		if($teamid == ''){
		$teamid = '0';
		}
		/* OST Changes */
		$this->_teamid = $teamid;
		$id = JFactory::getApplication()->input->get('id','0');
		$this->_id = $id;

		$playerid = JFactory::getApplication()->input->get('playerid', 0, 'INT');
		$this->_playerid = $playerid;

		/* set the team name; */
		$teamname = getTeamName($this->_teamid);
		$this->_teamname = $teamname;

		/* set the admin team ids; */
		$adminteams = array();
		$this->_db->setQuery("SELECT id, name, defaultteam, divisionid FROM #__fullcourt_team WHERE FIND_IN_SET('{$user->id}', adminids)");
		$adminteams = $this->_db->loadObjectList();
		$this->_adminteams = $adminteams;
	}

	function getTeam() {
		if (!$this->_team) {
			$row =  JTable::getInstance('team', 'Table');
			$row->load($this->_id);

			if ($row->id) {
			}
			else {
				$row->published = 1;
				$row->defaultteam = 1;
				$row->leagueid = 0;
			}

			$lists = array();
			/* get a list of leagues and divisions - teams will need to be assigned to a league; */
			$this->_leagues = getLeaguesAsSelectListAjax('leagueid', $row->leagueid);
			$this->_divisions = getDivisionsAsSelectList($row->leagueid, 'divisionid', $row->divisionid);
			$this->_team = $row;
		}
		return $this->_team;
	}

	function getRoster() {
		if (!$this->_roster) {
			$sql = "SELECT a.*, CONCAT(a.lname, ', ', a.fname, ' ', a.mname) AS fullname, t.name AS teamname, u.name AS editor, l.name AS league, d.division AS division FROM #__fullcourt_player AS a LEFT JOIN #__fullcourt_team AS t ON a.teamid = t.id LEFT JOIN #__fullcourt_league AS l ON t.leagueid = l.id LEFT JOIN #__fullcourt_division AS d ON t.divisionid = d.id  LEFT JOIN #__users AS u ON u.id = a.checked_out WHERE (a.teamid = {$this->_teamid}) GROUP BY a.id ORDER BY a.teamid, a.lname";
			$this->_db->setQuery($sql);
			$rows = $this->_db->loadObjectList();
			$this->_roster = $rows;
		}
		return $this->_roster;
	}

	function getPlayer() {
		if (!$this->_player) {
			$row =  JTable::getInstance('player', 'Table');
			$row->load($this->_id);

			if ($row->id) {
				/* no id... */
			}
			else {
				$row->published = 1;
				$row->defaultteam = 1;
				$row->leagueid = 0;
			}

			$this->_player = $row;

			$lists = array();

			$lists['teams'] = getTeamsAsSelectListLong('teamid', $this->_player->teamid);
			$lists['positions'] = getPositionsAsSelectList('positions', $this->_player->positions);
			$lists['birthdate'] = getDateAsSelectList('bmonth', 'bday', 'byear', $this->_player->birthdate);

			$this->_lists = $lists;
		}
		return $this->_player;
	}

	function getSchedule() {
		if (!$this->_schedule) {
			$sql = "SELECT a.*,
				h.name AS hometeam,
				v.name AS visitingteam,
				t.description AS gametype,
				l.name AS fieldname,
				l.city AS city,
				l.state AS state,
				s.description AS status,
				n.name AS season
				FROM #__fullcourt_schedule AS a
				LEFT JOIN #__fullcourt_season AS n ON a.season = n.id
				LEFT JOIN #__fullcourt_team AS h ON a.hometeam = h.id
				LEFT JOIN #__fullcourt_team AS v ON a.visitingteam = v.id
				LEFT JOIN #__fullcourt_gametype AS t ON a.gametype = t.id
				LEFT JOIN #__fullcourt_location AS l ON a.location = l.id
				LEFT JOIN #__fullcourt_gamestatus AS s ON a.status = s.id
				WHERE (a.visitingteam = {$this->_teamid} OR a.hometeam = {$this->_teamid})
				ORDER BY a.gamedatetime DESC";
			$this->_db->setQuery($sql);
			$rows = $this->_db->loadObjectList();
			$this->_schedule = $rows;
			//error_log("DEBUG: admin.php: getSchedule(): rows: ". $rows . "\n",3,"/home/bingniea/public_html/upgrade/public_html/logs/debug.log");
		}
		return $this->_schedule;
	}

	function getGame() {
		if (!$this->_game) {
			$row =  JTable::getInstance('schedule', 'Table');
			$row->load($this->_id);

			if ($row->id) {
				$row->gdate = date('Y-m-d', strtotime($row->gamedatetime));
				$row->gtime = date('H:i', strtotime($row->gamedatetime));
				$row->gendtime = date('H:i', strtotime($row->gameendtime));
				//error_log("DEBUG: admin.php: getGame(): row->gtime: ". $row->gtime . " row->gameendtime: " . $row->gameendtime . "\n",3,"/home/bingniea/public_html/upgrade/public_html/logs/debug.log");
				if ($row->gtime == '00:00') {
					$row->gtime = "TBA";
				}
			} else {
				/* get the default league id; */
				$this->_db->setQuery("SELECT id AS defaultleague FROM #__fullcourt_league WHERE (isdefault = 1)");
				$default = $this->_db->loadResult();
				$row->gdate = date('Y-m-d');
				$row->gtime = "TBA";
				$row->published = 1;
				$row->leagueid = $default;
			}

			/* get a list of leagues and divisions - schedules need to be assigned to a league; */
			$lists = array();
			$lists['seasons'] = getSeasonsAsSelectList('season', $row->season);
			$lists['hometeam'] = getTeamsAsSelectListLong('hometeam', $row->hometeam);
			$lists['visitingteam'] = getTeamsAsSelectListLong('visitingteam', $row->visitingteam);
			$lists['gametypes'] = getGameTypesAsSelectList('gametype', $row->gametype);
			$lists['locations'] = getLocationsAsSelectListAJAX('location', $row->location);
			$lists['fields'] = getParkFieldsSelectList($row->location, 'field', $row->field);
			$lists['gamestatus'] = getGameStatusAsSelectList('status', $row->status);
			$this->_lists = $lists;

			$this->_game = $row;
		}
		//error_log("DEBUG: admin.php: Returing ... \n",3,"/home/bingniea/public_html/upgrade/public_html/logs/debug.log");
		return $this->_game;
	}

	function getLocations() {
		if (!$this->_locations) {
			$sql = "SELECT a.* FROM #__fullcourt_location AS a GROUP BY a.id ORDER BY a.name";
			$this->_db->setQuery($sql);
			$rows = $this->_db->loadObjectList();
			$this->_locations = $rows;
		}
		return $this->_locations;
	}

	function getLocation() {
		if (!$this->_location) {
			$row =  JTable::getInstance('location', 'Table');
			$row->load($this->_id);

			if ($row->id) {
			} else {
				$row->fieldnames = array();
				$row->published = 1;
			}

			@$fieldnames = (array)unserialize($row->fieldnames);
			if (!$fieldnames) {
				$fieldnames = array();
			}

			$return = JFactory::getApplication()->input->get('return', '');
			$this->_return = $return;
			$this->_fieldnames = $fieldnames;
			$this->_location = $row;
		}
		return $this->_location;
	}

	function getGames() {
		if (!$this->_games) {
			$sql = "SELECT a.*, b.finalv, b.finalh,
				h.name AS hometeam,
				v.name AS visitingteam,
				t.description AS gametype,
				s.description AS status,
				n.name AS season
				FROM #__fullcourt_schedule AS a
				LEFT JOIN #__fullcourt_linescore AS b ON a.id = b.gameid
				LEFT JOIN #__fullcourt_season AS n ON a.season = n.id
				LEFT JOIN #__fullcourt_team AS h ON a.hometeam = h.id
				LEFT JOIN #__fullcourt_team AS v ON a.visitingteam = v.id
				LEFT JOIN #__fullcourt_gametype AS t ON a.gametype = t.id
				LEFT JOIN #__fullcourt_gamestatus AS s ON a.status = s.id
				WHERE (FIND_IN_SET(a.hometeam, '{$this->_teamid}') OR FIND_IN_SET(a.visitingteam, '{$this->_teamid}'))
				ORDER BY a.gamedatetime DESC";

			$this->_db->setQuery($sql);
			$rows = $this->_db->loadObjectList();
			$this->_games = $rows;
		}
		return $this->_games;
	}

	function getLinescore() {
		$db = $this->getDBO();
		$query = "SELECT * FROM #__fullcourt_linescore WHERE (gameid = {$this->_game->id})";
		$db->setQuery($query);
		$this->_boxscore = $db->loadObject();

		/* unserialize the OT for the visiting team; */
		$this->_boxscore->overtimev = unserialize($this->_boxscore->overtimev);

		/* unserialize the OT for the home team; */
		$this->_boxscore->overtimeh = unserialize($this->_boxscore->overtimeh);

		return $this->_boxscore;
	}

	function getPlayerStats() {
		$db = $this->getDBO();

		/* get the list of players for the team; */
		$query = "SELECT p.id, p.teamid, CONCAT(p.fname, ' ', p.lname) AS name FROM #__fullcourt_player AS p WHERE (p.id = {$this->_playerid})";
		$db->setQuery($query);
		$player = $db->loadObject();

		$query = "SELECT b.* FROM #__fullcourt_boxscore AS b WHERE (b.playerid = {$player->id} AND b.gameid = {$this->_game->id})";
		$db->setQuery($query);
		$game = $db->loadObject();

		$playerstats = new stdClass();

		if ($game) {
			$playerstats->id = $player->id;
			$playerstats->name = $player->name;
			$playerstats->gameid = $game->gameid;
			$playerstats->teamid = $game->teamid;
			$playerstats->seasonid = $game->seasonid;
			$playerstats->gs = $game->gs;
			$playerstats->pos = $game->pos;
			$playerstats->min = $game->min;
			$playerstats->fgm = $game->fgm;
			$playerstats->fga = $game->fga;
			$playerstats->threepm = $game->threepm;
			$playerstats->threepa = $game->threepa;
			$playerstats->ftm = $game->ftm;
			$playerstats->fta = $game->fta;
			$playerstats->oreb = $game->oreb;
			$playerstats->dreb = $game->dreb;
			$playerstats->ast = $game->ast;
			$playerstats->stl = $game->stl;
			$playerstats->blk = $game->blk;
			$playerstats->turnovers = $game->turnovers;
			$playerstats->pf = $game->pf;
			$playerstats->pts = $game->pts;
		}
		else {
			$playerstats->id = $player->id;
			$playerstats->name = $player->name;
			$playerstats->gameid = $this->_id;
			$playerstats->teamid = $player->teamid;
			$playerstats->seasonid = '';
			$playerstats->gs = '';
			$playerstats->pos = '';
			$playerstats->min = '';
			$playerstats->fgm = '';
			$playerstats->fga = '';
			$playerstats->threepm = '';
			$playerstats->threepa = '';
			$playerstats->ftm = '';
			$playerstats->fta = '';
			$playerstats->oreb = '';
			$playerstats->dreb = '';
			$playerstats->ast = '';
			$playerstats->stl = '';
			$playerstats->blk = '';
			$playerstats->turnovers = '';
			$playerstats->pf = '';
			$playerstats->pts = '';
		}

		$this->_playerstats = $playerstats;
		return $this->_playerstats;
	}

	function getBoxscore($teamid) {
		$db = $this->getDBO();
		$arr = array();

		/* get the list of players for the team; */
		$query = "SELECT p.id, CONCAT(p.lname, ', ', p.fname) AS name FROM #__fullcourt_player AS p WHERE (p.teamid = {$teamid}) ORDER BY name ASC";
		$db->setQuery($query);
		$players = $db->loadObjectList();
		foreach ($players as $player) {
			$playerstats = new stdClass();
			$query = "SELECT b.* FROM #__fullcourt_boxscore AS b WHERE (b.playerid = {$player->id} AND b.gameid = {$this->_game->id})";
			$db->setQuery($query);
			$game = $db->loadObject();
			if ($game) {
				$playerstats->id = $player->id;
				$playerstats->name = $player->name;
				$playerstats->gameid = $game->gameid;
				$playerstats->teamid = $game->teamid;
				$playerstats->seasonid = $game->seasonid;
				$playerstats->gs = $game->gs;
				$playerstats->pos = $game->pos;
				$playerstats->min = $game->min;
				$playerstats->fgm = $game->fgm;
				$playerstats->fga = $game->fga;
				$playerstats->threepm = $game->threepm;
				$playerstats->threepa = $game->threepa;
				$playerstats->ftm = $game->ftm;
				$playerstats->fta = $game->fta;
				$playerstats->oreb = $game->oreb;
				$playerstats->dreb = $game->dreb;
				$playerstats->ast = $game->ast;
				$playerstats->stl = $game->stl;
				$playerstats->blk = $game->blk;
				$playerstats->turnovers = $game->turnovers;
				$playerstats->pf = $game->pf;
				$playerstats->pts = $game->pts;
			}
			else {
				$playerstats->id = $player->id;
				$playerstats->name = $player->name;
				$playerstats->gameid = '';
				$playerstats->teamid = '';
				$playerstats->seasonid = '';
				$playerstats->gs = '';
				$playerstats->pos = '';
				$playerstats->min = '';
				$playerstats->fgm = '';
				$playerstats->fga = '';
				$playerstats->threepm = '';
				$playerstats->threepa = '';
				$playerstats->ftm = '';
				$playerstats->fta = '';
				$playerstats->oreb = '';
				$playerstats->dreb = '';
				$playerstats->ast = '';
				$playerstats->stl = '';
				$playerstats->blk = '';
				$playerstats->turnovers = '';
				$playerstats->pf = '';
				$playerstats->pts = '';
			}
			$arr[] = $playerstats;
		}

		$this->_boxscore = $arr;
		return $this->_boxscore;
	}

	function getVisitingteamRoster() {
		$db = $this->getDBO();
		$query = "SELECT id AS value, CONCAT(SUBSTR(fname, 1, 1), '. ', lname) AS text FROM #__fullcourt_player WHERE (teamid = {$this->_game->visitingteam})";
		$db->setQuery($query);
		$roster = $db->loadObjectList();
		return $roster;
	}

	function getHometeamRoster() {
		$db = $this->getDBO();
		$query = "SELECT id AS value, CONCAT(lname, ', ', SUBSTR(fname, 1, 1), '.') AS text FROM #__fullcourt_player WHERE (teamid = {$this->_game->hometeam}) ORDER BY text ASC";
		$db->setQuery($query);
		$roster = $db->loadObjectList();
		return $roster;
	}

	function getHometeam() {
		$db = $this->getDBO();
		$query = "SELECT * FROM #__fullcourt_team WHERE (id = {$this->_game->hometeam})";
		$db->setQuery($query);
		$this->_hometeam = $db->loadObject();
		return $this->_hometeam;
	}

	function getVisitingteam() {
		$db = $this->getDBO();
		$query = "SELECT * FROM #__fullcourt_team WHERE (id = {$this->_game->visitingteam})";
		$db->setQuery($query);
		$this->_visitingteam = $db->loadObject();
		return $this->_visitingteam;
	}

}

?>
