<?php
/**
* @author Fastball Productions
* @package com_fullcourt
* @copyright Copyright (C) 2009-2017 Fastball Productions
* @license http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
*/

defined( '_JEXEC' ) or die( 'Restricted access' );

jimport( 'joomla.application.component.model' );
jimport( 'joomla.application.component.helper' );

class ModelFullcourtAll extends JModelLegacy {
	var $_defaultleagueid = null;
	var $_defaultseasonid = null;
	var $_selectedleagueid = null;
	var $_selecteddivisionid = null;
	var $_selectedteamid = null;
	var $_leaguename = null;
	var $_teamname = null;
	var $_leagues = null;
	var $_divisions = null;
	var $_teams = null;
	var $_standings = null;
	var $_isfrontendadmin = null;
	var $_team = null;
	var $_teamlogo = null;

	function __construct() {
		parent::__construct();

		/* set the default league; */
		$defaultleagueid = getDefaultLeagueId();
		$this->_defaultleagueid = $defaultleagueid;
		/* set the default league; */
		$defaultseasonid = getDefaultSeasonId();
		$this->_defaultseasonid = $defaultseasonid;

		/* get the league id if it was passed in; */
		$selectedleagueid = JFactory::getApplication()->input->get('league', $this->_defaultleagueid, 'INT');
		$this->_selectedleagueid = $selectedleagueid;
		if ($this->_selectedleagueid == 0) {
			$this->_selectedleagueid = $this->_defaultleagueid;
		}
		/* get the division id if it was passed in; */
		$selecteddivisionid = JFactory::getApplication()->input->get('division', 0, 'INT');
		$this->_selecteddivisionid = $selecteddivisionid;
		/* get the team id if it was passed in; */
		$selectedteamid = JFactory::getApplication()->input->get('team', 0, 'INT');
		$this->_selectedteamid = $selectedteamid;
		/* set the league and team names; */
		$leaguename = getLeagueName($this->_selectedleagueid);
		$this->_leaguename = $leaguename;
		$teamname = getTeamName($this->_selectedteamid);
		$this->_teamname = $teamname;

		$admin = isFrontendAdmin($this->_selectedteamid);
		$this->_isfrontendadmin = $admin;
	}

	/* return a generic select list of leagues; */
	function getLeaguesList() {
		$list = array();
		$list[] = JHTML::_('select.option', '0', JText::_( 'COM_FULLCOURT_SELECT_LEAGUES' ), 'value', 'text' );
		$query = "SELECT l.id AS value, l.name AS text FROM #__fullcourt_league AS l WHERE l.published = 1 ORDER BY l.name";
		$results = $this->_getList($query, 0, 0);
		if (sizeof($results) > 0) {
			$list = array_merge($list, $results);
			$this->_leagues = JHTML::_('select.genericlist', $list, 'league', 'class="inputbox"', 'value', 'text', $this->_selectedleagueid);
		}
		return $this->_leagues;
	}

	/* return a generic select list of divisions; */
	function getDivisionsList() {
		$list = array();
		$list[] = JHTML::_('select.option', '0', JText::_( 'COM_FULLCOURT_SELECT_DIVISIONS' ), 'value', 'text' );
		$query = "SELECT d.id AS value, d.division AS text FROM #__fullcourt_division AS d LEFT JOIN #__fullcourt_team AS t ON d.id = t.divisionid WHERE (t.published = 1 AND t.divisionid IS NOT NULL AND d.leagueid = {$this->_selectedleagueid}) GROUP BY d.id ORDER BY d.division";
		$results = $this->_getList($query, 0, 0);

		if (sizeof($results) > 0) {
			$list = array_merge($list, $results);
			$this->_divisions = JHTML::_('select.genericlist', $list, 'division', 'class="inputbox" onchange="document.getElementById(\'gi-f1\').submit();"', 'value', 'text', $this->_selecteddivisionid);
		}
		return $this->_divisions;
	}

	/* return a generic select list of teams; */
	function getTeamsList() {
		$list = array();
		$list[] = JHTML::_('select.option', '0', JText::_( 'COM_FULLCOURT_SELECT_TEAMS' ), 'value', 'text' );
		if ($this->_selectedleagueid) {
			if ($this->_selecteddivisionid) {
				$query = "SELECT t.id AS value, t.name AS text FROM #__fullcourt_team AS t WHERE (t.defaultteam = 1 AND t.published = 1 AND t.leagueid = {$this->_selectedleagueid} AND t.divisionid = {$this->_selecteddivisionid}) ORDER BY t.name";
			}
			else {
				$query = "SELECT t.id AS value, t.name AS text FROM #__fullcourt_team AS t WHERE (t.defaultteam = 1 AND t.published = 1 AND t.leagueid = {$this->_selectedleagueid}) ORDER BY t.name";
			}
		}
		else {
			$query = "SELECT t.id AS value, t.name AS text FROM #__fullcourt_team AS t WHERE (t.defaultteam = 1 AND t.published = 1) ORDER BY t.name";
		}
		$results = $this->_getList($query, 0, 0);
		if (sizeof($results) > 0) {
			$list = array_merge($list, $results);
			$this->_teams = JHTML::_('select.genericlist', $list, 'team', 'class="inputbox" onchange="document.getElementById(\'gi-f1\').submit();"', 'value', 'text', $this->_selectedteamid);
		}
		return $this->_teams;
	}

	/* calculate and return league/division standings; */
	function getLeagueStandings() {
		if ($this->_selectedleagueid && $this->_selecteddivisionid) {
			$teams = getStandingsTeams($this->_selecteddivisionid, $this->_selectedleagueid, $this->_defaultseasonid);
			$standings = getStandingsStats($teams, $this->_defaultseasonid);
		}
		else if ($this->_selectedleagueid) {
			$standings = array();
			
			//echo "SELECT t.divisionid, d.ordering FROM #__fullcourt_team AS t LEFT JOIN #__fullcourt_division AS d ON t.divisionid = d.id WHERE (t.published = 1 AND t.defaultteam = 1 AND t.leagueid = {$this->_selectedleagueid} AND t.divisionid > 0) GROUP BY t.divisionid ORDER BY d.ordering";
			/* check if there are divisions configured within the league, if so calculate each division separately; */
			$this->_db->setQuery("SELECT t.divisionid, d.ordering FROM #__fullcourt_team AS t LEFT JOIN #__fullcourt_division AS d ON t.divisionid = d.id WHERE (t.published = 1 AND t.defaultteam = 1 AND t.leagueid = {$this->_selectedleagueid} AND t.divisionid > 0) GROUP BY t.divisionid ORDER BY d.ordering");
			$divisions = $this->_db->loadObjectList();

			/* if there are divisions configured use them; */
			if (sizeof($divisions) > 1) {
				foreach ($divisions as $division) {
					$teams = getStandingsTeams($division->divisionid, $this->_selectedleagueid, $this->_defaultseasonid);
					$standings = array_merge($standings, getStandingsStats($teams, $this->_defaultseasonid));
				}
			}
			else { /* not divisions so get the standings for all teams within the league; */
				$teams = getStandingsTeams(0, $this->_selectedleagueid, $this->_defaultseasonid);
				$standings = getStandingsStats($teams, $this->_defaultseasonid);
			}
		}
		$this->_standings = $standings;
		return $this->_standings;
	}

	function getTeam() {
		if (!$this->_team) {
			$query = "SELECT * FROM #__fullcourt_team WHERE (id = {$this->_selectedteamid})";
			$this->_db->setQuery($query);
			$this->_team = $this->_db->loadObject();
			return $this->_team;
		}
	}

}

?>